/**
 * $Id: Enigma.java,v 1.2 2009/06/12 08:54:41 oboehm Exp $
 *
 * Copyright (c) 2009 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 10.06.2009 by oliver (ob@aosd.de)
 */
package patterntesting.sample;

import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.SecretKeySpec;

import patterntesting.runtime.annotation.*;

/**
 * The Enigma was a cipher machine of the Germans during World War II
 * (see {@link http://en.wikipedia.org/wiki/Enigma_machine}).
 * This class is only a demo how to use the different cipher engines
 * provided by the JDK. And it is an example to show the use of the
 * @ProfileMe annotation.
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 10.06.2009
 * @version $Revision: 1.2 $
 * @see http://www.owasp.org/index.php/Using_the_Java_Cryptographic_Extensions
 * @see http://www.informit.com/guides/content.aspx?g=java&seqNum=31
 */
public class Enigma {
    
    /**
     * This is a very simple example of a crypt routine. It uses a simple
     * XOR with the given password. As far as I know this was the algorithm
     * on the VAX in the good old days as processor speed was measured in kHz.
     * <br/>
     * This is also an examples for @ProfileMe together with the @DontLogMe
     * annotation for the given password. Normally you should never a 
     * password in producution.
     * 
     * @param bytes input data which is crypted after the call
     * @param password the secret password with shouldn't appear in a log
     * @return the crypted data
     */
    @ProfileMe
    public static byte[] simpleCrypt(byte[] bytes, @DontLogMe String password) {
        byte[] key = password.getBytes();
        for (int i = 0; i < bytes.length; i++) {
            bytes[i] = (byte) (bytes[i] ^ key[i % key.length]);
        }
        return bytes;
    }
    
    /**
     * @param bytes
     * @param password
     * @param algorithm "DES", "DESede" or "TripleDES"
     * @return the crypted bytes
     * @throws GeneralSecurityException
     */
    @ProfileMe
    public static byte[] encrypt(byte[] bytes, @DontLogMe String password,
            String algorithm) throws GeneralSecurityException {
        return crypt(bytes, password, algorithm, Cipher.ENCRYPT_MODE);
    }

    /**
     * @param bytes
     * @param password
     * @param algorithm "DES", "DESede" or "TripleDES"
     * @return the crypted bytes
     * @throws GeneralSecurityException
     */
    @ProfileMe
    public static byte[] decrypt(byte[] bytes, @DontLogMe String password,
            String algorithm) throws GeneralSecurityException {
        return crypt(bytes, password, algorithm, Cipher.DECRYPT_MODE);
    }

    private static byte[] crypt(byte[] bytes, String password,
            String algorithm, int mode) throws GeneralSecurityException {
        SecretKey key = getSecretKey(password, algorithm);
        Cipher c = Cipher.getInstance(algorithm);
        c.init(mode, key);
        return c.doFinal(bytes);
    }

    /**
     * The different algorithm needs different password length:
     * <dl>
     *  <dt>"DES"</dt>
     *      <dd>exactly 8 characters</dd>
     *  <dt>"DESede" or "TripleDES"</dt>
     *      <dd>24 bytes</dd>
     * </dl>
     * 
     * @param password "DES", "DESede" or "TripleDES"
     * @param algorithm
     * @return
     */
    private static SecretKey getSecretKey(String password, String algorithm) {
        password += "xxxxxxxxxxxxxxxxxxxxxxxx";
        if (algorithm.equals("DES")) {
            password = password.substring(0, 8);
        } else if (algorithm.equals("DESede") || algorithm.equals("TripleDES")) {
            password = password.substring(0, 24);
        }
        return new SecretKeySpec(password.getBytes(), algorithm);
    }
    
}

/**
 * $Log: Enigma.java,v $
 * Revision 1.2  2009/06/12 08:54:41  oboehm
 * testCryptPerformanceTripleDES() removed to speed up JUnit test
 *
 * Revision 1.1  2009/06/11 21:05:51  oboehm
 * Math example replaced now by Enigma class
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-samples/src/main/java/patterntesting/sample/Enigma.java,v $
 */
